# -*- encoding: utf-8 -*-
# ! python2

from __future__ import (absolute_import, division, print_function, unicode_literals)

import io

from django.contrib.staticfiles import finders
from django.template import Library
from django.templatetags.static import static
from django.utils.safestring import mark_safe

from django_cbtp_email.errors import CssForEmailNotFoundError

register = Library()


@register.simple_tag()
def css_direct(css_path):
    """
    :type css_path: str
    """
    if not css_path.endswith(".css"):
        raise ValueError("c")

    result = finders.find(css_path)

    if not result:
        raise CssForEmailNotFoundError(
            "Cannot find \"{}\", searched locations: {}".format(css_path, finders.searched_locations))

    with io.open(result, mode="rt", encoding="utf-8") as the_file:  # rt == read text mode
        css_content = the_file.read()

    css_style = "<style type=\"text/css\">{0}</style>".format(css_content)

    return mark_safe(css_style)


@register.simple_tag(takes_context=True)
def static_direct(context, static_path):
    """
    :type static_path: basestring, str
    """
    if "request" in context:
        return static(static_path)

    return "file:///{}".format(finders.find(static_path))



# -*- encoding: utf-8 -*-
# ! python2

from __future__ import (absolute_import, division, print_function, unicode_literals)

import logging
import os
from abc import ABCMeta

import six

from .mail_funcs import send_mail

logger = logging.getLogger(__name__)


class Mailer(object):
    __metaclass__ = ABCMeta

    template = None
    context = {}
    to = None
    subject = None
    attachment = None

    def __init__(self, **kwargs):
        """
        Constructor. Called in the URLconf; can contain helpful extra
        keyword arguments, and other things.
        """
        # Go through keyword arguments, and either save their values to our
        # instance, or raise an error.
        for key, value in six.iteritems(kwargs):
            setattr(self, key, value)

    def log_send_mail_error(self, exception_to_log):
        logger.error(exception_to_log,
                     extra={
                         'stack': True,
                         'data': {
                             'self': self,
                         }
                     }
                     )

    def attach_file(self, file_path):
        if not os.path.isfile(file_path):
            raise ValueError("\"{}\" is not a valid file".format(file_path))

        self.attachment = file_path

    def send_message(self):
        to = self.get_recipients()

        try:
            send_mail(
                self.subject,
                self.template,
                context=self.context,
                to=to,
                attachment=self.attachment
            )
        except Exception as ex:
            self.log_send_mail_error(ex)
            raise

    def get_recipients(self):
        if not self.to:
            raise ValueError("Recipient of e-mail is not set.")

        return self.to if hasattr(self.to, '__iter__') else [self.to]



# -*- encoding: utf-8 -*-
# ! python2

from __future__ import (absolute_import, division, print_function, unicode_literals)

from django.apps import AppConfig


class MailingAppConfig(AppConfig):
    """
    Django app config, this will set human readable name in Django admin.
    """

    name = 'django_cbtp_email'
    verbose_name = "Django CBTP e-mail"



# coding=utf-8
import io

from setuptools import setup

try:
    from pypandoc import convert


    def read_md(file_name):
        # http://stackoverflow.com/a/23265673/752142
        return convert(file_name, 'rst')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")


    def read_md(file_name):
        try:
            return io.open(file_name, 'r', encoding='utf-8').read()
        except UnicodeDecodeError:
            return "Encoding problems with README.md"

# https://hynek.me/articles/sharing-your-labor-of-love-pypi-quick-and-dirty/
setup(
    name='django_cbtp_email',
    version='0.1.1',
    description=' Class-based mailer for Django using templates with inlined CSS.',

    # ########################################################################
    #
    # README.rst is generated from README.md:
    #
    # $ pandoc --from=markdown --to=rst README.md -o .tmp/README.rst
    #
    # ~ OR ~
    #
    # $ fab build
    # ########################################################################
    long_description=read_md('README.md'),

    url='https://github.com/illagrenan/django-cbtp-email',
    license='MIT',
    author='Vašek Dohnal',
    author_email='vaclav.dohnal@gmail.com',

    # The exclude makes sure that a top-level tests package doesn’t get
    # installed (it’s still part of the source distribution)
    # since that would wreak havoc.
    # find_packages(exclude=['tests*'])
    packages=['django_cbtp_email'],

    install_requires=['django', 'premailer', 'django-annoying'],
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'License :: OSI Approved :: MIT License',
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Environment :: Console',
        'Intended Audience :: Developers'
    ],
)



# -*- encoding: utf-8 -*-
# ! python2

from __future__ import (absolute_import, division, print_function, unicode_literals)

import logging
import os

from annoying.functions import get_config
from django.conf import settings
from django.core.mail import EmailMessage
from django.template.loader import render_to_string
from django.utils import translation
from premailer import Premailer

logger = logging.getLogger(__name__)


def send_mail(subject, template, context, to, from_email=settings.DEFAULT_FROM_EMAIL,
              template_variant=get_config('DEFAULT_TEMPLATE_TYPE', 'html'), attachment=None):
    """
    Render template and send it as a mail.

    Usage:
        > ctx = { 'some_var': True }
        > send_mail('E-mail subject', 'default_file_extensiontension/emails/bar', context=ctx, to=['joh.doe@example.com'])
    """
    template_path = os.path.normcase("{}.{}".format(template, template_variant))
    current_language = translation.get_language()

    if isinstance(to, str):
        to = [to]

    try:
        # TODO Allow to activate different language
        translation.activate(current_language)
        html_message = render_to_string(template_path, context)
    finally:
        translation.activate(current_language)

    premailer = Premailer(
        html_message,
        base_url=get_config('BASE_URL_FOR_EMAIL_LINK', None),
        base_path=settings.STATIC_ROOT
    )

    html_message = premailer.transform()

    mail = EmailMessage(settings.EMAIL_SUBJECT_PREFIX + subject, html_message, to=to, from_email=from_email)
    if template_variant == "html":
        mail.content_subtype = "html"

    if attachment:
        mail.attach_file(attachment)

    mail.send()
